/**************************************************************************************************
  Timer
**************************************************************************************************/
#include "Timer0.h"
#include "__Time.h"
#include "Common.h"
#include "USB_Driver.h"

// Variables
int TimerFlag = 0;  // Set by interrupt
TimeStruct ts1;
long Epoch;



// Timer Interrupt
void Timer0A_interrupt() iv IVT_INT_TIMER0A {
     TIMER_ICR_TATOCINT_bit = 1;         // Clear time-out timer A interrupt
     Epoch += 1;                         // Increment Date/Time by 1 second
     TimerFlag = 1;                      // Set timer flag
     //Net_Wireless_MCW1001_Time++ ;       // Increment Ethernet library counter
}

// Initialization
void TimerInit() {

     SYSCTL_RCGC1_TIMER0_bit = 1;        // Enable clock gating for timer module 0

     TIMER_CTL_TAEN_bit = 0;             // Disable timer
     TIMER0_CFG = 0;                     // Set 32-bit timer configuration
     TIMER0_TAMR |= 2;                   // Set periodic mode
     TIMER0_TAILR = Get_Fosc_kHz()*1000; // Set interval load
     NVIC_IntEnable(IVT_INT_TIMER0A);    // Enable timer interrupt
     TIMER_IMR_TATOIM_bit = 1;           // Enable time-out timer A interrupt
     TIMER_CTL_TAEN_bit = 1;             // Enable timer A

     ts1.ss = 0;  // Cold Start values
     ts1.mn = 0;
     ts1.hh = 0;
     ts1.md = 28;
     ts1.wd = 3;
     ts1.mo = 6;
     ts1.yy = 2012;

     Epoch = Time_dateToEpoch(&ts1);
}

// Timer processing is done if timer flag is set
void TimerCheck() {
     if (TimerFlag == 1) {
         TimerFlag = 0;  // Reset timer flag
         Time_epochToDate(Epoch, &ts1);

         irqhandler_timer();  // Last function in this module

/*
         // Update Date and Time only if Start Screen is showing
         if (DisplayFlag == 1) {
             UpdateDate();
             UpdateTime();
         }
*/
     //USB_PutQueue("Tick\r\n");
     }
}




/**************************************************************************************************
File:         Timers.c
Description:  Implements simple timer pool. No timer chaining
Resolution:   +0/-0.999 second

Author: Jari Lahti (jari.lahti@violasystems.com)
Version Info: 22.1.2004 - First version (JaL)
**************************************************************************************************/
//#include "timers.h"


// Variables
struct timers TimerPool[NUMTIMERS];


/**************************************************************************************************
Function:   init_timerpool
Parameters: none
Return val: void
Desc:       Initialize timer pool
**************************************************************************************************/
void init_timerpool(void) {
     unsigned short int i;

     for (i=0; i < NUMTIMERS; i++) {
          TimerPool[i].TimerField = 0;
          TimerPool[i].Free = TRUE;
     }
}

/**************************************************************************************************
Function:   get_timer
Parameters: none
Return val: unsigned short int - returns handle of free timer
Desc:       Gets free timer from the timer pool
**************************************************************************************************/
unsigned short int get_timer(void) {
     unsigned short int i;
     //unsigned short int FirstMatch;

     for (i=0; i < NUMTIMERS; i++) {
          if (TimerPool[i].Free == TRUE) {
              // We found a free timer!
              // Mark it reserved
              TimerPool[i].Free = FALSE;
              //FirstMatch = i;
              //return FirstMatch;  // Return Handle
              return i;
          }
     }

     // Error Check
     //UART1_PutQueue("TIMERS: No Timers, Resetting..\r\n");
     //RESET_SYSTEM();
     return (0);
}

/**************************************************************************************************
Function:   free_timer
Parameters: none
Return val: unsigned short int nbr - number
Desc:       Puts the given timer back to the timer pool
**************************************************************************************************/
void free_timer(unsigned short int nbr) {
     // Make a simple check
     if (nbr > (NUMTIMERS-1))
         return;

     TimerPool[nbr].Free = TRUE;
}

/**************************************************************************************************
Function:   set_timer
Parameters: unsigned short int nbr - number
            unsigned long int tout - time out
Return val: none
Desc:       Initializes given timer to given time-out
**************************************************************************************************/
void set_timer(unsigned short int nbr, unsigned long int tout) {
     //unsigned long int val;

     // Make a simple check
     if (nbr > (NUMTIMERS-1))
         return;

     if (TimerPool[nbr].Free == TRUE)
         return;

     // All OK
     //val = tout;

     // Normalize seconds to timer tics
     //TimerPool[nbr].TimerField = val;
     TimerPool[nbr].TimerField = tout;
}

/**************************************************************************************************
Function:   check_timer
Parameters: unsigned short int nbr - number
Return val: unsigned long int - returns the value of given timer
Desc:       Return the value of given timer
**************************************************************************************************/
unsigned long int check_timer(unsigned short int nbr) {
     return TimerPool[nbr].TimerField;
}

/**************************************************************************************************
Function:   irqhandler_timer
Parameters: void
Return val: void
Desc:       Call this function on timer interrupt
**************************************************************************************************/
void irqhandler_timer(void) {
     unsigned short int i;

     for (i=0; i<NUMTIMERS; i++) {
          if (TimerPool[i].Free == FALSE && TimerPool[i].TimerField != 0)
          TimerPool[i].TimerField--;
     }
}