/**************************************************************************************************
  UART0.c
  Custom UART driver for the Microchip MCW1001A chip.
  On the MikroMedia board for Stellaris M3 wire the following signals to the WiFi Plus Click board:
  MM board HDR2 pin 22 (U0Tx-PA1) to WiFi click board RX pin
  MM board HDR2 pin 21 (U0Rx-PA0) to WiFi click board TX pin
  MM board HDR2 pin 13 (PC5) to WiFi click board RST pin
  MM board HDR2 pin 25 (VCC-3.3) to WiFi click board +3.3V pin
  MM board HDR2 pin 26 (GND) to WiFi click board GND pin
  WiFi click board SCL pin (RTS#) to WiFi click board SDA pin (CTS#)
**************************************************************************************************/

// Include Files
#include "UART0.h"
#include "Buffers.h"
#include "Common.h"
#include "USB_Driver.h"
#include "WiFiPlus.h"


// Variables
char RxIn;                   // Receive FIFO
char RxOut;
char RxQueue[rxqsize];

char RcvBuf[RcvBufSize];     // Receive message buffer
unsigned int RcvBufPtr;      // Receive message character pointer

char TxIn;                   // Transmit pointer queue FIFO
char TxOut;
char *TxQueue[txqsize];
unsigned int Size[txqsize];

char *XmtBuf;                // Pointer to the current message being transmitted
unsigned int XmtBufPtr;      // Points to a byte in transmit buffer

// The receive message routine has 2 states
tPointerFcn UART0_RxFuncPtr = UART0_Rx1;  // Start in State 1

char RxState = 0;            // State machine state for receiving message packets
char MsgType0;               // Low order message type character
char MsgType1;               // High order message type character
char MsgDL0;                 // Low order message data length character
char MsgDL1;                 // High order message data length character
unsigned int RxMsgLength;    // Computed message length in bytes
unsigned int TxMsgLength;    // Computed message length in bytes
char ACK_Received = FALSE;   // Set to TRUE each time an ACK is received


// The transmit message routine has 2 states
tPointerFcn UART0_TxFuncPtr = UART0_Tx1;  // Start in state 1

extern WiFi_Properties WiFiParam;
extern WiFi_Events WiFiEvents;
extern WiFi_ScanResult WiFiScanResult;
extern unsigned short int NumScanRslts;
extern char ErrorType[2];
extern unsigned short int SocketHandle;
extern unsigned short int BindResult;
extern unsigned short int ConnectResult;
extern unsigned short int BacklogCount;
extern unsigned short int ConnectStatus;
extern char CurrentSSID[32];
extern char Config[8];
extern char ChanNumStr[8];

extern char UDP_RemoteHandle;
extern char UDP_RemoteIP[4];
extern unsigned int UDP_RemPort;
extern unsigned int UDP_RemByteCount;
extern char UDP_Buf[128];
extern char myAP_Key[32];
extern char AssignedIP[4];


/**************************************************************************************************
  Initialize UART0 */
void UART0Init(void) {
     UART0_RxQueueInit();
     UART0_TxQueueInit();

     UART0_Initialize();     // Use library routine
}

/**************************************************************************************************
  This routine sets UART0 baud rate and enables the receiver and transmitter */
void UART0_Initialize(void) {

     UART0_Init_Advanced(9600, 80000, _UART_8_BIT_DATA, _UART_NOPARITY, _UART_TWO_STOPBITS,
                         _UART_HIGH_SPEED, &_GPIO_MODULE_UART0_A01);

     Delay_ms(1000);
}
//-------------------------------------------------------------------------------------------------


// The receive portion operating in polling mode **************************************************
/**************************************************************************************************
  This routine initializes the receive buffer in and out pointers. */
void UART0_RxQueueInit(void) {
     RxIn = 0;
     RxOut = 0;
     RcvBufPtr = 0;
}

/**************************************************************************************************
  This routine enters a received character into the re-circulating buffer if any */
void UART0_Receive_Char(void) {
     if  (UART0_Data_Ready()) {
          RxQueue[RxIn++] = UART0_Read();
          if (RxIn > (rxqsize-1)) {
              RxIn = 0;
          }
     }
}

/**************************************************************************************************
  This routine retrieves a byte from the re-circulating buffer if present. Returns 0 if no data
  available. Returns 1 if a data byte has been transferred successfully. The pointer to the data
  byte is passed to the routine. */
char UART0_GetChar(char *rxbyte) {
     if (RxIn != RxOut){
        *rxbyte = RxQueue[RxOut++];
        if (RxOut > (rxqsize-1))
           RxOut = 0;
        return 1;
     }else{
        return 0;
     }
}

/**************************************************************************************************
  UART0_Rx State Machine. Call this function in main loop */
void UART0_Rx(void) {
     UART0_RxFuncPtr();    // Use the function pointer to go to a state
}

/**************************************************************************************************
  This routine extracts the Message Type, Data length, and Data from the message packet. */
void UART0_Rx1(void) {
     char rxchar;    // Character extracted from receive FIFO
     char *BufNum;

     // Read data if ready
     if  (UART0_GetChar(&rxchar)) {

//------- Header part of packet -------------------------------------------------------------------
          switch (RxState) {
              case 0: // Looking for header0 character
                  if (rxchar == 0x55)
                      RxState = 1;
                  else {
                      BufNum = GetBuffer();
                      sprintf(BufNum,"Extra char: %02X\r\n",rxchar);
                      USB_PutQueue(BufNum);
                  }
                  break;
              case 1: // Looking for header1 character
                  if (rxchar == 0xAA)
                      RxState = 2; // Expected
                  else {
                      USB_PutQueue("Incorrect header 1 character\r\n");
                      RxState = 0; // Unexpected
                  }
                  break;
              case 2: // Looking for message type 0 character (little-endian)
                  MsgType0 = rxchar;
                  RxState = 3;
                  break;
              case 3: // Looking for message type 1 character (little-endian)
                  MsgType1 = rxchar;
                  RxState = 4;
                  break;
              case 4: // Looking for message data length 0 character (little-endian)
                  MsgDL0 = rxchar;
                  RxState = 5;
                  break;
              case 5: // Looking for message data length 1 character (little-endian)
                  MsgDL1 = rxchar;

                  RxMsgLength = MsgDL0; // Compute message length
                  RxMsgLength |= (MsgDL1 << 8);

                  if (RxMsgLength == 0) {
                      if ((MsgType0 == 0) && (MsgType1 == 128))
                          RxState = 8; // ACK has no trailer according to the manual ??????
                      else
                          RxState = 7; // No data field
                  } else {
                      RcvBufPtr = 0;
                      RxState = 6; // Some data
                  }
                  break;
//----------- Data part of packet -----------------------------------------------------------------
              case 6: // Get the data
                  RcvBuf[RcvBufPtr++] = rxchar;
                  if (RcvBufPtr == RxMsgLength)
                      RxState = 7;
                  break;
//----------- Trailer part of packet --------------------------------------------------------------
              case 7: // Looking for trailer
                  if (rxchar == 0x45) {
                      //USB_PutQueue("Going to state 2\r\n");
                      UART0_RxFuncPtr = UART0_Rx2; // Go to Rx2 to process message
                  } else {
                      USB_PutQueue("Missing trailer character\r\n");
                  }
                  RxState = 0; // Finished receiving the packet
                  break;
//----------- ACK special processing --------------------------------------------------------------
              case 8: // ACK received
                  ACK_Received = TRUE;
                  USB_PutQueue("ACK received\r\n");
                  RxState = 0;
                  break;
          }
     }
}

/**************************************************************************************************
  UART0 State 2: Execute the received command and go back to State 1*/
void UART0_Rx2(void) {
     char *BufNum;
     unsigned int MT0, MT1, DL0, DL1;
     unsigned int i;
     unsigned int LengthSSID;
     //unsigned int RcvByteCount;

     USB_PutQueue("\r\nWiFi message received\r\n");

     MT0 = MsgType0;
     MT1 = MsgType1;
     DL0 = MsgDL0;
     DL1 = MsgDL1;
     BufNum = GetBuffer();         // Get a free buffer
     sprintf(BufNum,"MT0: %u, MT1: %u, DL0: %u, DL1: %u\r\n",MT0,MT1,DL0,DL1);
     USB_PutQueue(BufNum);         // Send the buffer out USB port

     switch(MT0) {
         case 0x01: // EVENT_MSG
             switch (RcvBuf[0]) {
                 case 0x08: // Event Type: Wi-Fi Connection Status Changed Event = 8
                     WiFiEvents.ConStatChng = TRUE;  // Post event
                     ConnectStatus = RcvBuf[1];
                     BufNum = GetBuffer();
                     switch (RcvBuf[1]) {
                         case 0x01: USB_PutQueue("Connection attempt succeeded\r\n"); break;
                         case 0x02: USB_PutQueue("Connection attempt failed\r\n"); break;
                         case 0x03: USB_PutQueue("Connection temporarily lost\r\n"); break;
                         case 0x04: USB_PutQueue("Connection reestablished\r\n"); break;
                         case 0x05: USB_PutQueue("Connection permanently lost\r\n"); break;
                     }
                     break;
                 case 0x09: // Event Type: Wi-Fi Scan Results Ready Event = 9
                     WiFiEvents.ScnRsltRdy = TRUE;  // Post event
                     NumScanRslts = RcvBuf[1];  // Save number of scan results
                     BufNum = GetBuffer();
                     sprintf(BufNum,"Number of scan results: %02X\r\n",RcvBuf[1]);
                     USB_PutQueue(BufNum);
                     break;
                 case 0x10: // Event Type: IP Address Assigned Event = 16
                     WiFiEvents.IPAddrAss = TRUE;
                     AssignedIP[0] = RcvBuf[2];
                     AssignedIP[1] = RcvBuf[3];
                     AssignedIP[2] = RcvBuf[4];
                     AssignedIP[3] = RcvBuf[5];
                     BufNum = GetBuffer();
                     sprintf(BufNum,"IP address assigned: %u.%u.%u.%u\r\n",RcvBuf[2],RcvBuf[3],RcvBuf[4],RcvBuf[5]);
                     USB_PutQueue(BufNum);
                     break;
                 case 0x1A: // Event Type: Ping Response Event = 26
                     WiFiEvents.PingResp = TRUE;
                     if (RcvBuf[1] == 0)
                         USB_PutQueue("Valid ping response was received\r\n");
                     else if (RcvBuf[1] == 1)
                         USB_PutQueue("Ping response time out\r\n");
                     BufNum = GetBuffer();
                     sprintf(BufNum,"Response time: %02X %02X\r\n",RcvBuf[2],RcvBuf[3]);
                     USB_PutQueue(BufNum);
                     break;
                 case 0x1B: // Event Type: Startup Event = 27
                     WiFiEvents.StartUp = TRUE;
                     if ((RcvBuf[1] & 0x01) != 0)
                          USB_PutQueue("Power on reset\r\n");
                     if ((RcvBuf[1] & 0x02) != 0)
                          USB_PutQueue("Brown out reset\r\n");
                     if ((RcvBuf[1] & 0x04) != 0)
                          USB_PutQueue("Idle reset\r\n");
                     if ((RcvBuf[1] & 0x08) != 0)
                          USB_PutQueue("Sleep reset\r\n");
                     if ((RcvBuf[1] & 0x10) != 0)
                          USB_PutQueue("Watchdog reset\r\n");
                     if ((RcvBuf[1] & 0x40) != 0)
                          USB_PutQueue("Software reset\r\n");
                     if ((RcvBuf[1] & 0x80) != 0)
                          USB_PutQueue("External reset\r\n");
                     BufNum = GetBuffer();
                     sprintf(BufNum,"WiComm-socket version: %02X %02X\r\n",RcvBuf[2],RcvBuf[3]);
                     USB_PutQueue(BufNum);
                     break;
                 case 0xFF: // Event Type: Error Event = 255
                     WiFiEvents.Error = TRUE;
                     ErrorType[0] = RcvBuf[2];
                     ErrorType[1] = RcvBuf[3];
                     BufNum = GetBuffer();
                     sprintf(BufNum,"Error event type: %02X %02X, Error data: %02X %02X\r\n",RcvBuf[2],RcvBuf[3],RcvBuf[4],RcvBuf[5]);
                     USB_PutQueue(BufNum);
                     break;
                 default:   // In case there is some event not described in the manual
                     WiFiEvents.Unknown = TRUE;
                     BufNum = GetBuffer();
                     sprintf(BufNum,"Unknown event: %u\r\n",RcvBuf[0]);
                     USB_PutQueue(BufNum);
                     break;
             }
             break;

         case 0x16: // SCAN_RESULT_MSG (22 RESPONSE)
             WiFiEvents.ScnRsltMsg = TRUE;

             LengthSSID = RcvBuf[6];
             for (i=0;i<LengthSSID;i++)
                  CurrentSSID[i] = RcvBuf[i+7];
             CurrentSSID[i] = 0;
             sprintf(ChanNumStr,"%02X",RcvBuf[56]);
             if ((RcvBuf[39] & 0x80) != 0) strcpy(Config,"WPA2");
             else if ((RcvBuf[39] & 0x40) != 0) strcpy(Config,"WPA");

             BufNum = GetBuffer();
             sprintf(BufNum,"BSSID: %02X %02X %02X %02X %02X %02X\r\n",RcvBuf[0],RcvBuf[1],RcvBuf[2],RcvBuf[3],RcvBuf[4],RcvBuf[5]);
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             LengthSSID = RcvBuf[6];
             for (i=0;i<LengthSSID;i++)
                  BufNum[i] = RcvBuf[i+7];
             BufNum[i] = 0;
             strcat(BufNum,"\r\n");
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"AP configuration: %02X\r\n",RcvBuf[39]);
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"Channel number: %02X\r\n",RcvBuf[56]);
             USB_PutQueue(BufNum);
             break;

         case 0x17: // SOCKET_CREATE_RESPONSE_MSG (23 RESPONSE)
             WiFiEvents.SockCreaResp = TRUE;
             SocketHandle = RcvBuf[0];
             switch (RcvBuf[0]) {
                 case 254: USB_PutQueue("Invalid socket\r\n"); break;
                 case 255: USB_PutQueue("Unknown socket type\r\n"); break;
                 default:
                     BufNum = GetBuffer();
                     sprintf(BufNum,"Valid socket handle: %u\r\n",RcvBuf[0]);
                     USB_PutQueue(BufNum);
                     break;
             }
             break;

         case 0x18: // SOCKET_BIND_RESPONSE_MSG (24 RESPONSE)
             WiFiEvents.SockBindResp = TRUE;
             BindResult = RcvBuf[2];
             BufNum = GetBuffer();
             sprintf(BufNum,"Local port: %02X %02X, Bind result: %02X\r\n",RcvBuf[0],RcvBuf[1],RcvBuf[2]);
             USB_PutQueue(BufNum);
             break;

         case 0x19: // SOCKET_CONNECT_RESPONSE_MSG (25 RESPONSE)
             WiFiEvents.SockConnResp = TRUE;
             ConnectResult = RcvBuf[0];
             switch (RcvBuf[0]) {
                 case 0: USB_PutQueue("Success\r\n"); break;
                 case 254: USB_PutQueue("Socket connect in process\r\n"); break;
                 case 255: USB_PutQueue("Socket connect failed\r\n"); break;
             }
             break;

         case 0x1A: // SOCKET_LISTEN_RESPONSE_MSG (26 RESPONSE)
             WiFiEvents.SockListenResp = TRUE;
             ConnectResult = RcvBuf[0];
             BacklogCount = RcvBuf[1];
             switch (RcvBuf[0]) {
                 case 0: USB_PutQueue("Success\r\n"); break;
                 case 254: USB_PutQueue("Socket connect in process\r\n"); break;
                 case 255: USB_PutQueue("Socket connect failed\r\n"); break;
             }
             break;

         case 0x1B: // SOCKET_ACCEPT_RESPONSE_MSG (27 RESPONSE)
             WiFiEvents.SockAccptResp = TRUE;

             BufNum = GetBuffer();
             sprintf(BufNum,"Socket handle: %u\r\n",RcvBuf[0]);
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"Remote port: %02X %02X\r\n",RcvBuf[2],RcvBuf[3]);
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"Remote IP address: %u.%u.%u.%u\r\n",RcvBuf[4],RcvBuf[5],RcvBuf[6],RcvBuf[7]);
             USB_PutQueue(BufNum);
             break;

         case 0x1C: // SOCKET_SEND_RESPONSE_MSG (28 RESPONSE)
             WiFiEvents.SockSendResp = TRUE;

             BufNum = GetBuffer();
             sprintf(BufNum,"Socket byte count sent: %02X %02X\r\n",RcvBuf[0],RcvBuf[1]);
             USB_PutQueue(BufNum);
             break;

         case 0x1D: // SOCKET_RECEIVE_RESPONSE_MSG (29 RESPONSE)
             WiFiEvents.SockRcvResp = TRUE;
             UDP_RemoteHandle = RcvBuf[0];     // Destination socket handle
             UDP_RemPort = RcvBuf[2];          // Remote port
             UDP_RemPort |= (RcvBuf[3] << 8);

             BufNum = GetBuffer();
             sprintf(BufNum,"Socket handle: %u\r\n",RcvBuf[0]);
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"Receive byte count: %02X %02X\r\n",RcvBuf[2],RcvBuf[3]);
             USB_PutQueue(BufNum);
             break;

         case 0x1E: // SOCKET_SEND_TO_RESPONSE_MSG (30 RESPONSE)
             WiFiEvents.SockSendToResp = TRUE;

             BufNum = GetBuffer();
             sprintf(BufNum,"Byte count sent: %u\r\n",RcvBuf[0]);
             USB_PutQueue(BufNum);
             break;

         case 0x1F: // SOCKET_RECV_FROM_RESPONSE_MSG (31 RESPONSE)
             WiFiEvents.SockRcvFromResp = TRUE;
             UDP_RemoteHandle = RcvBuf[0];     // Destination socket handle
             UDP_RemPort = RcvBuf[2];          // Remote port
             UDP_RemPort |= (RcvBuf[3] << 8);
             UDP_RemoteIP[0] = RcvBuf[4];
             UDP_RemoteIP[1] = RcvBuf[5];
             UDP_RemoteIP[2] = RcvBuf[6];
             UDP_RemoteIP[3] = RcvBuf[7];
             UDP_RemByteCount = RcvBuf[20];    // Message byte count
             UDP_RemByteCount |= (RcvBuf[21] << 8);
             for (i=0;i<UDP_RemByteCount;i++)
                  UDP_Buf[i] = RcvBuf[i+22];
             UDP_Buf[i] = 0;

             BufNum = GetBuffer();
             sprintf(BufNum,"Socket handle: %u\r\n",RcvBuf[0]);
             USB_PutQueue(BufNum);   // Display the socket handle

             BufNum = GetBuffer();
             sprintf(BufNum,"Port number of sender: %02X %02X\r\n",RcvBuf[2],RcvBuf[3]);
             USB_PutQueue(BufNum);   // Display sender's port number

             BufNum = GetBuffer();
             sprintf(BufNum,"IP address of sender: %u.%u.%u.%u\r\n",RcvBuf[4],RcvBuf[5],RcvBuf[6],RcvBuf[7]);
             USB_PutQueue(BufNum);   // Display sender's IP address

             BufNum = GetBuffer();
             sprintf(BufNum,"Byte count received: %02X %02X\r\n",RcvBuf[20],RcvBuf[21]);
             USB_PutQueue(BufNum);   // Display the byte count

             BufNum = GetBuffer();
             sprintf(BufNum,"%02X%02X%02X%02X",UDP_Buf[0],UDP_Buf[1],UDP_Buf[2],UDP_Buf[3]);
             USB_PutQueue(BufNum);   // Display the 1st 4 bytes of UDP buffer in hex
             USB_PutQueue(UDP_Buf);  // Display the UDP buffer directly
             break;

         case 0x20: // SOCKET_ALLOCATE_RESPONSE_MSG (32 RESPONSE)
             WiFiEvents.SockAllocResp = TRUE;

             switch (RcvBuf[0]) {
                 case 0: USB_PutQueue("Socket allocation successful\r\n"); break;
                 case 254: USB_PutQueue("Total buffer allocation size exceeded\r\n"); break;
                 case 255: USB_PutQueue("Maximum number of sockets exceeded\r\n"); break;
             }
             BufNum = GetBuffer();
             sprintf(BufNum,"Allocated socket count: %02X\r\n",RcvBuf[1]);
             USB_PutQueue(BufNum);
             break;

         case 0x30: // NETWORK_STATUS_RESPONSE_MSG (48 RESPONSE)
             WiFiEvents.NetStatResp = TRUE;  // Set network response flag
             for (i=0;i<6;i++)
                  WiFiParam.MACAddress[i] = RcvBuf[i+1];
             for (i=0;i<4;i++)
                  WiFiParam.IPAddress[i] = RcvBuf[i+7];
             for (i=0;i<4;i++)
                  WiFiParam.NetworkMask[i] = RcvBuf[i+23];
             for (i=0;i<4;i++)
                  WiFiParam.GatewayAddress[i] = RcvBuf[i+39];
             WiFiParam.NetworkStatus = RcvBuf[55];

             BufNum = GetBuffer();
             sprintf(BufNum,"Reserved: %02X\r\n",RcvBuf[0]); // Reserved
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"MAC address: %02X %02X %02X %02X %02X %02X\r\n",RcvBuf[1],RcvBuf[2],
             RcvBuf[3],RcvBuf[4],RcvBuf[5],RcvBuf[6]); // MAC address (6 bytes)
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"IP address: %02X%02X%02X%02X%02X%02X%02X%02X",
             RcvBuf[7],RcvBuf[8],RcvBuf[9],RcvBuf[10],RcvBuf[11],RcvBuf[12],RcvBuf[13],RcvBuf[14]); // IP address (first 8 bytes)
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"%02X%02X%02X%02X%02X%02X%02X%02X\r\n",
             RcvBuf[15],RcvBuf[16],RcvBuf[17],RcvBuf[18],RcvBuf[19],RcvBuf[20],RcvBuf[21],RcvBuf[22]); // IP address (last 8 bytes)
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"Network mask: %02X%02X%02X%02X%02X%02X%02X%02X",
             RcvBuf[23],RcvBuf[24],RcvBuf[25],RcvBuf[26],RcvBuf[27],RcvBuf[28],RcvBuf[29],RcvBuf[30]); // Network mask (first 8 bytes)
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"%02X%02X%02X%02X%02X%02X%02X%02X\r\n",
             RcvBuf[31],RcvBuf[32],RcvBuf[33],RcvBuf[34],RcvBuf[35],RcvBuf[36],RcvBuf[37],RcvBuf[38]); // Network mask (last 8 bytes)
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"Gateway addr: %02X%02X%02X%02X%02X%02X%02X%02X",
             RcvBuf[39],RcvBuf[40],RcvBuf[41],RcvBuf[42],RcvBuf[43],RcvBuf[44],RcvBuf[45],RcvBuf[46]); // Gateway address (first 8 bytes)
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"%02X%02X%02X%02X%02X%02X%02X%02X\r\n",
             RcvBuf[47],RcvBuf[48],RcvBuf[49],RcvBuf[50],RcvBuf[51],RcvBuf[52],RcvBuf[53],RcvBuf[54]); // Gateway address (last 8 bytes)
             USB_PutQueue(BufNum);

             BufNum = GetBuffer();
             sprintf(BufNum,"Network status: %02X\r\n",RcvBuf[55]); // Network status: 0-Not connected static IP, 1-Connected static IP, 2-Not connected DHCP, 3-Connected DHCP
             USB_PutQueue(BufNum);
             break;

         case 0x31: // WPAKEY_RESPONSE_MSG (49 RESPONSE)
             WiFiEvents.WpaKeyResp = TRUE;
             for (i=0;i<32;i++)
                  myAP_Key[i] = RcvBuf[i];

             BufNum = GetBuffer();
             sprintf(BufNum,"%02X%02X%02X%02X%02X%02X%02X%02X\r\n",RcvBuf[0],RcvBuf[1],RcvBuf[2],RcvBuf[3],RcvBuf[4],RcvBuf[5],RcvBuf[6],RcvBuf[7]);
             USB_PutQueue(BufNum);
             BufNum = GetBuffer();
             sprintf(BufNum,"%02X%02X%02X%02X%02X%02X%02X%02X\r\n",RcvBuf[8],RcvBuf[9],RcvBuf[10],RcvBuf[11],RcvBuf[12],RcvBuf[13],RcvBuf[14],RcvBuf[15]);
             USB_PutQueue(BufNum);
             BufNum = GetBuffer();
             sprintf(BufNum,"%02X%02X%02X%02X%02X%02X%02X%02X\r\n",RcvBuf[16],RcvBuf[17],RcvBuf[18],RcvBuf[19],RcvBuf[20],RcvBuf[21],RcvBuf[22],RcvBuf[23]);
             USB_PutQueue(BufNum);
             BufNum = GetBuffer();
             sprintf(BufNum,"%02X%02X%02X%02X%02X%02X%02X%02X\r\n",RcvBuf[24],RcvBuf[25],RcvBuf[26],RcvBuf[27],RcvBuf[28],RcvBuf[29],RcvBuf[30],RcvBuf[31]);
             USB_PutQueue(BufNum);
             break;

         case 0x32: // GPIO_RESPONSE_MSG (50 RESPONSE)
             WiFiEvents.GpioResp = TRUE;

             BufNum = GetBuffer();
             sprintf(BufNum,"%02X,%02X\r\n",RcvBuf[0],RcvBuf[1]);
             USB_PutQueue(BufNum);
             break;

         default:
             BufNum = GetBuffer(); // In case there is some response not described in the manual
             sprintf(Bufnum,"Unknown message type: %u %u\r\n",MT0,MT1);
             USB_PutQueue(BufNum);
             break;
     }
     //USB_PutQueue("Going to state 1\r\n");
     UART0_RxFuncPtr = UART0_Rx1;  // Go back to state 1
}
//-------------------------------------------------------------------------------------------------




// The transmitter portion operating in polling mode **********************************************
/**************************************************************************************************
  This routine initializes the transmit buffer queue in and out pointers */
void UART0_TxQueueInit(void){
     TxIn = 0;
     TxOut = 0;
     XmtBufPtr = 0;
}

/**************************************************************************************************
  This routine makes an entry into the transmit queue of pointers and updates the in pointer to
  the next available location. */
void UART0_PutQueue(char *msg, unsigned int msgsize) {
     Size[TxIn] = msgsize;     // Put the message Size into FIFO
     TxQueue[TxIn++] = msg;    // Put the message pointer into FIFO
     if (TxIn > (txqsize-1))   // Adjust FIFO in pointer
         TxIn = 0;
}

/**************************************************************************************************
  UART0_Tx State Machine. Call this function in main loop */
void UART0_Tx(void) {
     UART0_TxFuncPtr();    // Use the function pointer to go to a state
}

/**************************************************************************************************
  This routine checks for a data packet in the queue and sets up a pointer to the packet. */
void UART0_Tx1(void) {
     if (TxIn != TxOut) {
         TxMsgLength = Size[TxOut];    // Get message size
         XmtBuf = TxQueue[TxOut++];    // Get message pointer
         if (TxOut > (txqsize-1))      // Update FIFO out pointer
             TxOut = 0;
         XmtBufPtr = 0;                // Reset byte pointer to 0
         UART0_TxFuncPtr = UART0_Tx2;  // Go to state 2
     }
}
/**************************************************************************************************
  This routine sends a data packet one byte at a time out the transmitter. */
void UART0_Tx2(void) {
     if (UART0_Tx_Idle()) {                     // If transmitter ready
         UART0_Write(XmtBuf[XmtBufPtr++]);      // Put 1 byte in transmit register
         if (XmtBufPtr == TxMsgLength) {        // Check if at end of message
             USB_PutQueue("Message sent\r\n");
             UART0_TxFuncPtr = UART0_Tx1;       // Go back to state 1
         }
     }
}