/**************************************************************************************************
  WiFiPlus.h
  Custom WiFi code for the Microchip MCW1001A chip
**************************************************************************************************/
#ifndef WIFIPLUS_H
#define WIFIPLUS_H


// Defines
#define UDP 0
#define TCP 1

#define USE_PASSWORD 1 /* Using password takes over 30 seconds to connect. After connecting for the
first time with the password, use the GET_CP_WPAKEY_MSG command to retrieve the 32 bit KEY then
program it into the SET_CP_SECURITY_WPA_MSG_BINARY packet. Set the USE_PASSWORD to 0 and recompile. */

#define USE_DHCP 1   // Set to 0 for static IP address




typedef struct {
    char MACAddress[6];         // My MAC address
    char IPAddress[4];          // My IP address
    char NetworkMask[4];        // Network mask
    char GatewayAddress[4];     // Gateway IP address
    char NetworkStatus;         // Network status
} WiFi_Properties;

typedef struct {
    // Events
    char ConStatChng;           // Connection status changed event
    char ScnRsltRdy;            // Scan result ready event
    char IPAddrAss;             // IP address assigned event
    char PingResp;              // Ping response event
    char StartUp;               // Startup event
    char Error;                 // Error event
    char Unknown;               // Unknown event
    // Responses
    char ScnRsltMsg;            // Scan result flag
    char SockCreaResp;          // Socket create response flag
    char SockBindResp;          // Socket bind response flag
    char SockConnResp;          // Socket connect response flag
    char SockListenResp;        // Socket listen response flag
    char SockAccptResp;         // Socket accept response flag
    char SockSendResp;          // Socket send response flag
    char SockRcvResp;           // Socket receive response flag
    char SockSendToResp;        // Socket send to response flag
    char SockRcvFromResp;       // Socket receive from response flag
    char SockAllocResp;         // Socket allocate response flag
    char NetStatResp;           // Network status response flag
    char WpaKeyResp;            // WPA key response flag
    char GpioResp;              // GPIO response flag
} WiFi_Events;

// This structure is not used
typedef struct {
    char BSSID[6];              // Basic service set identification (BSSID)
    char SSIDLength;            // Service set identification (SSID) Length - Number of valid SSID characters that follow
    char SSID[32];              // Service set identification (SSID)
    char APConfig;              // AP configuration (8 bits). This byte is defined as follows:
                                /* Bit 7 - WPA2
                                           0: AP does not support WPA2.
                                           1: AP supports WPA2.
                                   Bit 6 - WPA
                                           0: AP does not support WPA.
                                           1: AP supports WPA.
                                   Bit 5 - Preamble
                                           0: AP transmitting with short preamble.
                                           1: AP transmitting with long preamble.
                                   Bit 4 - Privacy
                                           0: AP is open (no security).
                                           1: AP using security; if neither WPA or WPA2 is set then security is WEP.
                                   Bit 3 - Res: Reserved
                                   Bit 2 - Res: Reserved
                                   Bit 1 - Res: Reserved
                                   Bit 0 - IE : 1 if AP broadcasting one or more information elements, otherwise 0.*/
    int  beaconPeriod;          // Beacon Period - Network beacon interval
    int  ATIMWindow;            // ATIM (Announcement Traffic Indication Message) Window (only valid if BSS Type is ad hoc)
    char basicRates[8];         // Basic Rates, each 1 byte, in the following format:
                                /* Bit 7 - Basic rates set bit
                                           0: Rate is not part of the "basic rates set".
                                           1: Rate is part of the "basic rates set".
                                   Bits 6:0 - Multiple of 500 kbps giving the supported rate. For example, a value of 2 (2x500kbps),
                                              indicates that 1 Mbps is a supported rate. A value of 4 in this field indicates a 2 Mbps rate (4x500 kbps).*/
    char RSSI;                  // RSSI of received beacon or probe response
    char basicRatesListLength;  // Number of basic rates that are valid in above basic rate list
    char DTIMPeriod;            // DTIM (Delivery Traffic Identification Maps) period (part of TIM element)
    char BSSType;               // 1: Infrastructure, 2: ad hoc
    char Channel;               // Channel number
} WiFi_ScanResult;


// Function Prototype
void WiFiPlusInit(void);
void WiFiPlusHwReset(void);
void WiFiSwReset(void);
void WiFiSetIpAddr(void);
void WiFiSetNetworkMask(void);
void WiFiSetGatewayAddr(void);
void WiFiSetMacAddr(void);
void WiFiSetArpTime(void);
void WiFiSetNetworkMode(void);
void WiFiSetCpSsid(void);
void WiFiSetRegionalDomain(void);
void WiFiSetChannelList(void);
void WiFiSetListRetryCount(void);
void WiFiSetSecurityWpa(void);
void WiFiSetSecurityWpaBinary(void);
void WiFiScanStart(void);
void WiFiConnect(void);
void WiFiDisconnect(void);
void WiFiPingSend(void);

void WiFiSocketCreate(char SocType);
void WiFiSocketClose(char Handle);
void WiFiSocketBind(char Handle, unsigned int LocalPort);
void WiFiSocketConnect(char Handle);
void WiFiSocketListen(char Handle);
void WiFiSocketAccept(char Handle);
void WiFiSocketSend(char Handle, unsigned short *Data);
void WiFiSocketReceive(char Handle, unsigned int DataLength);
void WiFiSocketSendTo(char Handle, char *Data);
void WiFiSocketReceiveFrom(char Handle, unsigned int DataLength);


void WiFiGetVersion(void);
void WiFiGetNetworkStatus(void);
void WiFiGetWpaKey(void);
void WiFiScanGetResult(unsigned short int Index);


// WiFi Application State Machine Function Prototypes
void WiFiApp(void);
void WiFiState0(void);

// Sequence to fetch network status
void WiFiState1(void);
void WiFiState2(void);

// Sequence to scan for nearby access points
void WiFiState4(void);
void WiFiState5(void);
void WiFiState6(void);
void WiFiState7(void);
void WiFiState8(void);
void WiFiState9(void);
void WiFiState10(void);

// Sequence to set MAC addresses, IP address, Network mask, and Gateway IP address
void WiFiState15(void);
void WiFiState16(void);
void WiFiState17(void);
void WiFiState18(void);
void WiFiState19(void);
void WiFiState20(void);
void WiFiState21(void);
void WiFiState22(void);
void WiFiState23(void);
void WiFiState24(void);
void WiFiState25(void);

void WiFiState30(void);
void WiFiState31(void);

void WiFiState35(void);
void WiFiState36(void);

void WiFiState40(void);
void WiFiState41(void);

void WiFiState45(void);
void WiFiState46(void);

void WiFiState50(void);
void WiFiState51(void);

void WiFiState55(void);
void WiFiState56(void);

void WiFiState60(void);
void WiFiState61(void);

void WiFiState65(void);
void WiFiState66(void);

void WiFiState67(void);
void WiFiState68(void);

void WiFiState70(void);
void WiFiState71(void);

void WiFiState74(void);
void WiFiState75(void);

void WiFiState80(void);
void WiFiState81(void);
void WiFiState82(void);

void WiFiState90(void);
void WiFiState91(void);
void WiFiState92(void);
void WiFiState93(void);
void WiFiState94(void);
void WiFiState95(void);
void WiFiState96(void);
void WiFiState97(void);

void WiFiState100(void);
void WiFiState101(void);
void WiFiState102(void);
void WiFiState103(void);
void WiFiState104(void);
void WiFiState105(void);
void WiFiState106(void);
void WiFiState107(void);
void WiFiState108(void);
void WiFiState109(void);
void WiFiState110(void);
void WiFiState111(void);
void WiFiState112(void);
void WiFiState113(void);

void WiFiState120(void);
void WiFiState121(void);
void WiFiState122(void);
void WiFiState123(void);
void WiFiState124(void);

void WiFiState130(void);
void WiFiState131(void);
void WiFiState132(void);
void WiFiState133(void);


void WiFiState666(void);  // The Dreaded Evil Error Event

void TimeoutCheck(unsigned int TimeoutState);



void ClearAllLabels(void);
void UpdateLabel4(char *msg);
void UpdateLabel5(char *msg);
void UpdateLabel6(char *msg);
void UpdateLabel7(char *msg);
void UpdateLabel8(char *msg);
void UpdateLabel9(char *msg);
void UpdateLabel10(char *msg);
void UpdateLabel11(char *msg);
void UpdateLabel12(char *msg);
void UpdateLabel13(char *msg);
void UpdateLabel14(char *msg);
void UpdateLabel15(char *msg);
void UpdateLabel16(char *msg);
void UpdateLabel17(char *msg);
void UpdateLabel18(char *msg);
void UpdateLabel19(char *msg, unsigned int Color);

void EnableButton2(void);
void EnableButton3(void);
void EnableButton4(void);

void EraseButton2(void);
void EraseButton3(void);
void EraseButton4(void);



#endif